@@ -1,11 +1,13 @@
 """
-The standard library for the Deltaflow language.
+The standard library for the Deltaflow language contains:
 
-Contents:
+- Primitives for generation, re-routing, and collection of data:
+  :py:func:`make_generator`
+  :py:func:`make_splitter`
+  :py:class:`StateSaver`
 
-- Hardware Abstraction Layer (HAL) routines
-- Primitives
-- Quantum Simulators
+- Routines for the Hardware Abstraction Layer (HAL) and Quantum Simulators,
+  which are best described in :doc:`examples/examples`.
 """
 
 from deltalanguage.lib.hal import (HardwareAbstractionLayerNode,
