@@ -1,4 +1,5 @@
 from enum import Enum
+from typing import Iterable, List
 
 from deltalanguage.data_types import DUInt, DSize, DArray
 
@@ -8,10 +9,7 @@ from deltalanguage.data_types import DUInt, DSize, DArray
 
 
 class Opcode(Enum):
-    """Operational codes for HAL.
-
-    .. note:: Not user-facing class.
-    """
+    """Operational codes for HAL."""
 
     NOP = 0
     STATE_PREPARATION = 1
@@ -48,8 +46,6 @@ class Opcode(Enum):
 class Shifts(Enum):
     """
     .. TODO:: Missing description.
-
-    .. note:: Not user-facing class.
     """
 
     ARG = 16
@@ -60,8 +56,6 @@ class Shifts(Enum):
 class Masks(Enum):
     """
     .. TODO:: Missing description.
-
-    .. note:: Not user-facing class.
     """
 
     QUBIT_INDEX = 65535
@@ -73,8 +67,6 @@ class Masks(Enum):
 def command_creator(op: str, argument=0, qubit=0) -> DUInt(DSize(32)):
     """Helper function to create HAL commands.
 
-    .. TODO:: Eventually will be inside DType classes, so can remove later.
-
     Parameters
     ----------
     op : str
@@ -93,32 +85,59 @@ def command_creator(op: str, argument=0, qubit=0) -> DUInt(DSize(32)):
         | (argument << Shifts.ARG.value) | qubit
 
 
-def measurement_unpacker(bitcode: DUInt(DSize(32)), qubits: DArray) -> DArray:
+def command_unpacker(cmd: DUInt(DSize(32))):
+    """Helper function to unpack HAL commands.
+
+    Parameters
+    ----------
+    cmd : DUInt(DSize(32))
+        32-bit HAL command.
+
+    Returns
+    -------
+    op : str
+        Name of opcode.
+    argument : int
+        Integer representation of argument value.
+    qubit : int
+        Integer representation of qubit address.
+    """
+    op_mask = sum(map(lambda n: 2**n, range(Shifts.OPCODE.value, 32)))
+    op = (cmd & op_mask) >> Shifts.OPCODE.value
+    # We pass strings around rather than Opcode elements, so we use
+    # this to get the reverse mapping from the Opcode class
+    # This is semi-safe, as we don't have repeated names for operations
+    op = Opcode._value2member_map_[op]._name_
+
+    arg_mask = sum(map(lambda n: 2**n,
+                       range(Shifts.ARG.value, Shifts.OPCODE.value)))
+    arg = (cmd & arg_mask) >> Shifts.ARG.value
+
+    qubit_mask = sum(map(lambda n: 2**n, range(0, Shifts.VALIDS.value)))
+    qubit = (cmd & qubit_mask)
+    return (op, arg, qubit)
+
+
+def measurement_unpacker(bitcode: DUInt(DSize(32)), qubits: Iterable) -> List:
     """Helper function to convert 32-bit status result from HAL into an array
     of measurements for given qubit indices.
 
-    .. TODO::
-        Replace ``DArray`` by either ``object`` or specify its type and
-        length.
-
-
     Parameters
     ----------
     bitcode : DUInt(DSize(32))
         32-bit measurement status from HAL.
-    qubits : DArray
+    qubits : Iterable
         List of qubits for which the measurement result will be returned.
 
+    Returns
+    -------
+    List
+        List of measurement results for the specified qubits.
+
     Raises
     ------
     ValueError
         If not all valid flags are 1.
-
-
-    Returns
-    -------
-    DArray
-        List of measurement results for the specified qubits.
     """
 
     # split status bitcode into measurements (first 16 bits) and
@@ -129,7 +148,8 @@ def measurement_unpacker(bitcode: DUInt(DSize(32)), qubits: DArray) -> DArray:
     # print('1', measurements)
 
     if valids != Masks.VALIDS.value << Shifts.VALIDS.value:
-        raise ValueError("Invalid measurement!")
+        raise ValueError(f"Invalid measurement!, {hex(valids)}, "
+                         f"{hex(Masks.VALIDS.value << Shifts.VALIDS.value)}")
 
     measurements_list = []
 
@@ -138,3 +158,29 @@ def measurement_unpacker(bitcode: DUInt(DSize(32)), qubits: DArray) -> DArray:
 
     # print('2', measurements_list)
     return measurements_list
+
+
+def measurement_creator(measurement_array, qubits):
+    """Helper function to convert 32-bit status result from HAL into an array
+    of measurements for given qubit indices.
+
+    Parameters
+    ----------
+    measurement_array
+        List of measurement results for the specified qubits.
+    qubits : Iterable
+        List of qubits for which the measurement result will be returned.
+
+    Returns
+    -------
+    bitcode : DUInt(DSize(32))
+        32-bit measurement status from HAL.
+    """
+    packed_measurements = 0
+    for i in qubits:
+        measurement = measurement_array.pop()
+        packed_measurements |= 2**i if measurement else 0
+
+    packed_measurements |= Masks.VALIDS.value
+
+    return packed_measurements
