@@ -1,24 +1,25 @@
 """
 Primitives for examples, tests, and just general use.
 """
+import json
 from copy import deepcopy
 from typing import Callable, Iterable, Type, Union
 
-from ..data_types import (BaseDeltaType, NoMessage,
-                          delta_type, make_forked_return)
+from ..data_types import (BaseDeltaType, Void,
+                          delta_type, make_forked_return,
+                          DComplex, DArray, DRecord, DTuple)
 from ..runtime import DeltaRuntimeExit
 from ..wiring import (DeltaBlock,
                       DeltaMethodBlock,
                       Interactive,
-                      PyInteractiveNode,
+                      PythonNode,
                       InteractiveProcess)
 
 
 def make_generator(val: Union[object, Iterable],
                    reps: int = None,
                    verbose: bool = False) -> InteractiveProcess:
-    """Returns a generator implemented via
-    :py:class:`PyInteractiveNode<deltalanguage.wiring.PyInteractiveNode>` that
+    """Used to create a generator node that
     produces a series of messages of the same data type.
 
     This generator is useful for checking the robustness of the graph vs
@@ -36,8 +37,6 @@ def make_generator(val: Union[object, Iterable],
         Number of repeated messages.
         If ``reps`` is ``None`` then ``val`` must be iterable.
         If ``reps`` is an integer then `val` is sent out this number of times.
-        
-        are sent out one by one.
     verbose : bool
         If ``True`` prints the status.
 
@@ -49,15 +48,13 @@ def make_generator(val: Union[object, Iterable],
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib import StateSaver, make_generator
-        >>> from deltalanguage.wiring import DeltaGraph, Interactive
-        >>> from deltalanguage.runtime import DeltaPySimulator
+        >>> import deltalanguage as dl
 
         # Generate 5 integers
-        >>> generator = make_generator(10, reps=5)
+        >>> generator = dl.lib.make_generator(10, reps=5)
 
         # Receive 5 integers and send their sum
-        >>> @Interactive({"a": int}, int)
+        >>> @dl.Interactive({"a": int}, int)
         ... def accumulator(node):
         ...     memory = []
         ...     for _ in range(5):
@@ -66,15 +63,15 @@ def make_generator(val: Union[object, Iterable],
         ...     node.send(sum(memory))
 
         # Save the result
-        >>> s = StateSaver(int, verbose=True)
+        >>> s = dl.lib.StateSaver(int, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     generator_out = generator.call()
         ...     accumulator_out = accumulator.call(a=generator_out)
         ...     s.save_and_exit(accumulator_out) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 50
 
@@ -82,15 +79,15 @@ def make_generator(val: Union[object, Iterable],
 
     .. code-block:: python
 
-        >>> generator = make_generator([1, 2, 3, 4, 5])
+        >>> generator = dl.lib.make_generator([1, 2, 3, 4, 5])
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     generator_out = generator.call()
         ...     accumulator_out = accumulator.call(a=generator_out)
         ...     s.save_and_exit(accumulator_out) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 15
     """
@@ -108,7 +105,7 @@ def make_generator(val: Union[object, Iterable],
         vals_to_send = (deepcopy(val) for _ in range(reps))
 
     @Interactive({}, elem_type)
-    def generator(node: PyInteractiveNode):
+    def generator(node: PythonNode):
         for v in vals_to_send:
             if verbose:
                 print(f"sending {val}")
@@ -121,10 +118,11 @@ def make_generator(val: Union[object, Iterable],
 def make_splitter(t: Union[Type, BaseDeltaType],
                   reps: int,
                   allow_const=True) -> Callable:
-    """Returns a splitter node, which returns multiple copies of incoming
-    messages via individual outputs.
+    """Used to create a splitter node, which sends multiple copies of
+    incoming messages to different nodes via individual outputs.
 
-    Names of outputs go as `out0`, `out1`, `out2`, ...
+    Names of output ports go in ascending order as ``out0``, ``out1``,
+    ``out2``, etc. The total number of outputs is equal to ``reps``.
 
     Examples
     --------
@@ -132,25 +130,23 @@ def make_splitter(t: Union[Type, BaseDeltaType],
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib import StateSaver, make_splitter
-        >>> from deltalanguage.wiring import DeltaBlock, DeltaGraph
-        >>> from deltalanguage.runtime import DeltaPySimulator
+        >>> import deltalanguage as dl
 
-        >>> @DeltaBlock()
+        >>> @dl.DeltaBlock()
         ... def adder(a: int, b: int) -> int:
         ...     return a + b
 
-        >>> state_saver = StateSaver(int, verbose=True)
+        >>> state_saver = dl.lib.StateSaver(int, verbose=True)
 
-        >>> splitter = make_splitter(int, 2)
+        >>> splitter = dl.lib.make_splitter(int, 2)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     splitter_out = splitter(10)
         ...     adder_out = adder(splitter_out.out0, splitter_out.out1)
         ...     state_saver.save_and_exit(adder_out) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 20
     """
@@ -165,6 +161,24 @@ def make_splitter(t: Union[Type, BaseDeltaType],
     return _splitter
 
 
+class DeltaJsonEncoder(json.JSONEncoder):
+    def default(self, obj):
+        obj_type = delta_type(obj)
+        if isinstance(obj_type, DComplex):
+            return {"real": obj.real, "imaginary": obj.imag}
+        elif isinstance(obj_type, (DArray, DTuple)):
+            return [self.default(o) for o in obj]
+        elif isinstance(obj_type, DRecord):
+            props = obj.__annotations__.keys()
+            d = {p: self.default(getattr(obj, p)) for p in props}
+            return d
+
+        # Assume it's encodable; if we go back to JSONEncoder.default
+        # this just returns a type error. See:
+        # https://docs.python.org/3/library/json.html#json.JSONEncoder.default
+        return obj
+
+
 class StateSaver:
     """Class used for saving states via various methods.
 
@@ -180,6 +194,13 @@ class StateSaver:
         Used for the conditional blocks, see examples.
     verbose : bool
         If ``True`` prints a status on node's activation.
+    filename : str
+        A filepath in which to save results, incrementally.
+
+        If provided, this file will be appended with
+        line-based JSON representation of the data saved.
+        Note: The writing to the file is done when
+        ``save`` is called is called.
 
     Examples
     --------
@@ -187,13 +208,11 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib import make_generator, StateSaver
-        >>> from deltalanguage.wiring import DeltaGraph
-        >>> from deltalanguage.runtime import DeltaPySimulator
+        >>> import deltalanguage as dl
 
-        >>> s = StateSaver(int, verbose=True)
+        >>> s = dl.lib.StateSaver(int, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save(42) # doctest:+ELLIPSIS
         save...
 
@@ -201,11 +220,11 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit(42) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 42
 
@@ -214,13 +233,13 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> s = StateSaver(int, condition=lambda x: x>10, verbose=True)
+        >>> s = dl.lib.StateSaver(int, condition=lambda x: x>10, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit_if(42) # doctest:+ELLIPSIS
         save_and_exit_if...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 42
 
@@ -229,16 +248,16 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> gen = make_generator([42])
-        >>> s0 = StateSaver(int, verbose=True)
-        >>> s1 = StateSaver(bool, verbose=True)
+        >>> gen = dl.lib.make_generator([42])
+        >>> s0 = dl.lib.StateSaver(int, verbose=True)
+        >>> s1 = dl.lib.StateSaver(bool, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     status = s0.save_and_ack(gen.call())
         ...     s1.save_and_exit(status) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 42
         saving True
@@ -247,16 +266,16 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> gen = make_generator([42])
-        >>> s0 = StateSaver(int, verbose=True)
-        >>> s1 = StateSaver(int, verbose=True)
+        >>> gen = dl.lib.make_generator([42])
+        >>> s0 = dl.lib.StateSaver(int, verbose=True)
+        >>> s1 = dl.lib.StateSaver(int, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     message = s0.transfer(gen.call())
         ...     s1.save_and_exit(message) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 42
         saving 42
@@ -265,16 +284,17 @@ class StateSaver:
 
     .. code-block:: python
 
-        >>> gen = make_generator([3, 8])
-        >>> s0 = StateSaver(int, condition=lambda x: x%2==0, verbose=True)
-        >>> s1 = StateSaver(int, verbose=True)
+        >>> gen = dl.lib.make_generator([3, 8])
+        >>> s0 = dl.lib.StateSaver(int, condition=lambda x: x%2==0,
+        ...                        verbose=True)
+        >>> s1 = dl.lib.StateSaver(int, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     message = s0.transfer_if(gen.call())
         ...     s1.save_and_exit(message) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 3
         saving 8
@@ -288,13 +308,15 @@ class StateSaver:
 
     def __init__(self, t: Union[Type, BaseDeltaType] = object,
                  condition=None,
-                 verbose=False):
+                 verbose=False,
+                 filename=None):
         self.saved = []
         self.condition = condition
         self.verbose = verbose
+        self.filename = filename
 
         @DeltaBlock(allow_const=False)
-        def save(val: t) -> NoMessage:
+        def save(val: t) -> Void:
             """Save input of any type.
 
             Parameters
@@ -306,7 +328,7 @@ class StateSaver:
         self.save = save
 
         @DeltaBlock(allow_const=False)
-        def save_and_exit(val: t) -> NoMessage:
+        def save_and_exit(val: t) -> Void:
             """Save input of any type and exit.
 
             Parameters
@@ -319,7 +341,7 @@ class StateSaver:
         self.save_and_exit = save_and_exit
 
         @DeltaBlock(allow_const=False)
-        def save_and_exit_if(val: t) -> NoMessage:
+        def save_and_exit_if(val: t) -> Void:
             """Save input of any type and exit if the condition is fulfilled.
 
             Parameters
@@ -396,3 +418,7 @@ class StateSaver:
         if self.verbose:
             print(f"saving {val}")
         self.saved.append(val)
+
+        if self.filename is not None:
+            with open(self.filename, "a") as f:
+                f.write(json.dumps(val, cls=DeltaJsonEncoder) + "\n")
