@@ -1,10 +1,7 @@
-"""Module for defining the decorators for making python functions into
-DeltaBlocks and DeltaMethods
-"""
+"""User facing decorators for building DeltaGraph nodes."""
 
 from __future__ import annotations
 from functools import wraps
-from inspect import _empty, signature
 import logging
 from typing import (TYPE_CHECKING,
                     Callable,
@@ -18,92 +15,23 @@ from typing import (TYPE_CHECKING,
 import attr
 
 from deltalanguage.data_types import (BaseDeltaType,
-                                      DOptional,
+                                      DeltaIOError,
                                       DeltaTypeError,
+                                      DOptional,
                                       ForkedReturn,
-                                      NoMessage,
+                                      Void,
                                       as_delta_type)
 from ._delta_graph import DeltaGraph
-from ._node_classes.interactive_node import (InteractiveFuncType,
-                                             PyInteractiveNode)
-from ._node_classes.real_nodes import PythonNode
+from ._node_classes.real_nodes import (as_node,
+                                       get_func_in_params_out_type,
+                                       PythonNode)
+from ._node_classes.node_bodies import PyInteractiveBody
 from ._node_factories import py_method_node_factory, py_node_factory
 
 if TYPE_CHECKING:
     from deltalanguage.wiring import Latency
 
 
-def get_func_in_params_out_type(
-    a_func: Callable,
-    is_method: bool,
-    node_key: Optional[str] = None
-) -> Tuple[OrderedDict[str, Union[BaseDeltaType, DOptional]],
-           Union[BaseDeltaType, ForkedReturn]]:
-    """Helper function to extract input and output types of a node function.
-
-    Parameters
-    ----------
-    a_func : Callable
-        The function to analyse.
-    is_method : bool
-        Flag to specify if function is a class method.
-    node_key : Optional[str]
-        Keyword argument used for providing the node to the block, included for
-        some logic purposes.
-
-    Returns
-    -------
-    OrderedDict[str, Union[BaseDeltaType, DOptional]]
-        Types of the in parameters.
-    Union[BaseDeltaType, ForkedReturn]
-        Type of the output the node to be made.
-
-    Raises
-    ------
-    TypeError
-        Raised if either the input or output types aren't specified in the
-        function signature.
-    """
-    func_args = signature(a_func).parameters
-    out_type = signature(a_func).return_annotation
-
-    in_params = OrderedDict()
-    for i, (arg_name, arg_param) in enumerate(func_args.items()):
-
-        # first argument should always be 'self' for a method
-        if i == 0 and is_method:
-            continue
-
-        if arg_param.annotation == _empty:
-            raise TypeError(
-                "Must specify the type of argument " +
-                f"'{arg_name}' as annotation in " +
-                f"function '{a_func.__name__}'"
-            )
-
-        if node_key and arg_param.annotation == PythonNode:
-            continue
-
-        delta_type_in = as_delta_type(arg_param.annotation)
-        if not isinstance(delta_type_in, (BaseDeltaType, DOptional)):
-            raise DeltaTypeError(f"Unsupported type={arg_param.annotation}")
-
-        in_params[arg_name] = delta_type_in
-
-    if out_type == _empty:
-        raise TypeError(
-            "Must specify the return type of " +
-            f"function '{a_func.__name__}' as an annotation"
-        )
-
-    delta_type_out = as_delta_type(out_type)
-    if delta_type_out is not NoMessage:
-        if not isinstance(delta_type_out, (BaseDeltaType, ForkedReturn)):
-            raise DeltaTypeError(f"Unsupported type={out_type}")
-
-    return in_params, delta_type_out
-
-
 def DeltaBlock(
     name: str = None,
     allow_const: bool = True,
@@ -115,7 +43,7 @@ def DeltaBlock(
     """Decorator to turn a function to a block for use in
     :py:class:`DeltaGraph`.
 
-    If evaluated in the context of :py:class:`DeltaGraph` it will
+    If called in the context of :py:class:`DeltaGraph` it will
     return a stateless node, which means that the output of such a node is
     fully determined by its inputs; exactly as it happens in functional
     programming.
@@ -127,6 +55,8 @@ def DeltaBlock(
     .. warning::
         If a node does not have compulsory inputs then it will be evaluated
         continuesly and this can significantly slow down a runtime simulator.
+        If node does not have optional inputs consider using
+        ``allow_const=True`` to improve performance.
 
     Parameters
     ----------
@@ -162,11 +92,9 @@ def DeltaBlock(
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib import StateSaver
-        >>> from deltalanguage.runtime import DeltaPySimulator
-        >>> from deltalanguage.wiring import DeltaBlock, DeltaGraph
+        >>> import deltalanguage as dl
 
-        >>> @DeltaBlock()
+        >>> @dl.DeltaBlock()
         ... def foo(a: int) -> int:
         ...     return a + 5
 
@@ -178,14 +106,14 @@ def DeltaBlock(
 
     .. code-block:: python
 
-        >>> s = StateSaver(object, verbose=True)
+        >>> s = dl.lib.StateSaver(object, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     foo_out = foo(5)
         ...     s.save_and_exit(foo_out) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 10
 
@@ -193,7 +121,7 @@ def DeltaBlock(
 
     .. code-block:: python
 
-        >>> @DeltaBlock()
+        >>> @dl.DeltaBlock()
         ... def bar(a: int, b: int) -> int:
         ...     return a*b
 
@@ -202,11 +130,11 @@ def DeltaBlock(
 
     .. code-block:: python
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit(bar(a=9, b=9)) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 81
 
@@ -271,16 +199,17 @@ def DeltaMethodBlock(
     """Decorator to turn a class method to a block for use in
     :py:class:`DeltaGraph`.
 
-    If evaluated in the context of :py:class:`DeltaGraph` it will
+    If called in the context of :py:class:`DeltaGraph` it will
     return a node that can have an internal state stored in the instance of
-    the target class. THus the output determined by not only the inputs but
+    the target class. Thus the output determined by not only the inputs but
     also by the internal state that can change.
 
     .. warning::
         The internal state is a very powerful concept that makes the Deltaflow
         language significantly more expressive, but also can lead to
-        non-deterministic results. Please refer to the tutorial
-        (TODO add the link) that cover covers this.
+        non-deterministic results. Please refer to
+        `Non-deterministic state history <tutorials/state_history.html>`_
+        that cover this in detail.
 
     The node is evaluated when all compulsory inputs are provided.
     By default each input is compulsory, in order to make it optional use
@@ -320,16 +249,14 @@ def DeltaMethodBlock(
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib import StateSaver
-        >>> from deltalanguage.runtime import DeltaPySimulator
-        >>> from deltalanguage.wiring import DeltaBlock, DeltaGraph
+        >>> import deltalanguage as dl
 
         >>> class MyClass:
         ...
         ...     def __init__(self, x):
         ...         self.x = x
         ...
-        ...     @DeltaMethodBlock()
+        ...     @dl.DeltaMethodBlock()
         ...     def bar(self, a: int) -> int:
         ...         return self.x + a
 
@@ -342,13 +269,13 @@ def DeltaMethodBlock(
 
     .. code-block:: python
 
-        >>> s = StateSaver(object, verbose=True)
+        >>> s = dl.lib.StateSaver(object, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit(my_obj.bar(5)) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 15
 
@@ -359,7 +286,7 @@ def DeltaMethodBlock(
 
         >>> my_obj.x = 15
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 20
 
@@ -426,22 +353,23 @@ class InteractiveProcess:
 
     Attributes
     ----------
-    proc : InteractiveFuncType
+    proc : Callable[[PythonNode], None]
         Class instance this node operates on.
     arg_types : Dict[str, Type]
         The types of the in parameters to the node to be made.
-    return_type : Type
+    out_type : Type
         The type of the output for the node to be made.
     name : str
         The name of the node to be made.
     in_port_size : int
         The maximum size of the node's in ports. If 0 then unlimited size.
     lvl : int
-        Logging level for the node. By default logging.ERROR.
+        Logging level for the node.
+        By default only error logs are displayed.
     """
-    proc: InteractiveFuncType = attr.ib()
+    proc: Callable[[PythonNode], None] = attr.ib()
     arg_types: Dict[str, Type] = attr.ib()
-    return_type: Type = attr.ib()
+    out_type: Type = attr.ib()
     name: str = attr.ib()
     in_port_size: int = attr.ib(default=0)
     lvl: int = attr.ib(default=logging.ERROR)
@@ -454,21 +382,36 @@ class InteractiveProcess:
             raise ValueError(
                 "Please only use keyword arguments for interactive nodes."
             )
+
         graph = DeltaGraph.current_graph()
-        new_node = PyInteractiveNode(graph, self, **kwargs)
+        my_body = PyInteractiveBody(self.proc)
+
+        kw_in_nodes = {name: as_node(arg, graph)
+                       for (name, arg) in kwargs.items()}
+
+        new_node = PythonNode(graph,
+                              [my_body],
+                              self.arg_types,
+                              [],
+                              kw_in_nodes,
+                              out_type=self.out_type,
+                              name=self.name,
+                              lvl=self.lvl,
+                              in_port_size=self.in_port_size)
         return new_node
 
 
 def Interactive(in_params: Dict[str, Type],
                 out_type: Type,
-                name=None,
+                name: str = None,
                 in_port_size: int = 0,
                 lvl: int = logging.ERROR) -> Callable[..., InteractiveProcess]:
     """Decorator to turn a function of a special type (see examples)
-    to a block for use in :py:class:`DeltaGraph`.
+    to an interactive node for use in :py:class:`DeltaGraph`.
 
-    The function used with this decorator is not meant to be on its own in
-    contrast with :py:class:`DeltaBlock` and :py:class:`DeltaMethodBlock`.
+    The function used with this decorator is not meant to be used on its own
+    (i.e. outside of the graph) in contrast with
+    :py:class:`DeltaBlock` and :py:class:`DeltaMethodBlock`.
 
     The types of inputs and outputs have to be defined using ``in_params`` and
     ``out_type``. By default each input is compulsory, in order to make it
@@ -478,18 +421,25 @@ def Interactive(in_params: Dict[str, Type],
     Unlike nodes created via :py:class:`DeltaBlock` and
     :py:class:`DeltaMethodBlock`, the interactive node is evaluated only once.
     Thus it's in the user's responsibility to specify when and which
-    inputs and outputs are used. This is done via ``send`` and ``receive``
-    methods.
+    inputs and outputs are used.
+    This is done via ``send`` and ``receive`` methods and shown in examples
+    below.
 
-    The inputs and outputs can be received and sent simultaneously
-    or just partially, which gives a lot of flexibility in comparison with
+    An interactive node's inputs/outputs can be received/sent simultaneously
+    or partially, which gives a lot of flexibility in comparison with
     :py:class:`DeltaBlock` and :py:class:`DeltaMethodBlock`.
     For instance, if a compulsory input is expected then by calling ``receive``
-    on this input the node execution will be frozen until the input is provided.
+    on this input the node execution will be frozen until the input is
+    provided.
     On the other hand, calling ``receive`` for an optional input will not
     freeze the node and simply use ``None`` if no input is provided at
     this instance of time.
 
+    An interactive node should use at least one input or output to communicate
+    with other nodes, otherwise runtime simulators, e.g.
+    :py:class:`DeltaPySimulator<deltalanguage.runtime.DeltaPySimulator>`,
+    might be significantly slowed down by their schedulers.
+
     Parameters
     ----------
     in_params : Dict[str, Type]
@@ -508,16 +458,14 @@ def Interactive(in_params: Dict[str, Type],
     --------
     Let's look at the syntax first.
     Here is an example where all inputs are received simultaneously several
-    times, before the output is sent off, then the node becomes idle:
+    times, before the output is sent off once, then the node becomes idle:
 
     .. code-block:: python
 
-        >>> from deltalanguage.lib.primitives import StateSaver
-        >>> from deltalanguage.runtime import DeltaPySimulator
-        >>> from deltalanguage.wiring import Interactive, PyInteractiveNode
+        >>> import deltalanguage as dl
 
-        >>> @Interactive(in_params={"a": int, "b": int}, out_type=int)
-        ... def foo(node: PyInteractiveNode):
+        >>> @dl.Interactive(in_params={"a": int, "b": int}, out_type=int)
+        ... def foo(node: PythonNode):
         ...     internal_memory = 0
         ...
         ...     for i in range(10):
@@ -527,20 +475,22 @@ def Interactive(in_params: Dict[str, Type],
         ...     node.send(internal_memory)
 
     .. note::
-        :py:class:`PyInteractiveNode` is simply used to syntax highlight.
+        :py:class:`PythonNode` is used to help with syntax highlight.
+        For instance when using a linting tool you can access definitions of
+        `send` and `receive` methods and see their parameters.
 
     To use this node in the context of :py:class:`DeltaGraph` a special
     method ``node`` should be used:
 
     .. code-block:: python
 
-        >>> s = StateSaver(int, verbose=True)
+        >>> s = dl.lib.StateSaver(int, verbose=True)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit(foo.call(a=4, b=5)) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving 200
 
@@ -548,8 +498,8 @@ def Interactive(in_params: Dict[str, Type],
 
     .. code-block:: python
 
-        >>> @Interactive(in_params={"a": int, "b": int}, out_type=int)
-        ... def bar(node: PyInteractiveNode):
+        >>> @dl.Interactive(in_params={"a": int, "b": int}, out_type=int)
+        ... def bar(node: PythonNode):
         ...     internal_memory = 0
         ...
         ...     for i in range(10):
@@ -560,11 +510,11 @@ def Interactive(in_params: Dict[str, Type],
         ...
         ...     node.send(internal_memory)
 
-        >>> with DeltaGraph() as graph:
+        >>> with dl.DeltaGraph() as graph:
         ...     s.save_and_exit(bar.call(a=4, b=5)) # doctest:+ELLIPSIS
         save_and_exit...
 
-        >>> rt = DeltaPySimulator(graph)
+        >>> rt = dl.DeltaPySimulator(graph)
         >>> rt.run()
         saving -5
 
@@ -574,12 +524,12 @@ def Interactive(in_params: Dict[str, Type],
 
     .. code-block:: python
 
-        >>> from deltalanguage.data_types import make_forked_return
+        >>> import deltalanguage as dl
 
-        >>> TwoIntsT, TwoIntsC = make_forked_return({'x': int, 'y': int})
+        >>> TwoIntsT, TwoIntsC = dl.make_forked_return({'x': int, 'y': int})
 
-        >>> @Interactive(in_params={"a": int}, out_type=TwoIntsT)
-        ... def baz(node: PyInteractiveNode):
+        >>> @dl.Interactive(in_params={"a": int}, out_type=TwoIntsT)
+        ... def baz(node: PythonNode):
         ...     for i in range(10):
         ...         a = node.receive("a")
         ...         if a%3 == 0:
@@ -589,7 +539,12 @@ def Interactive(in_params: Dict[str, Type],
         ...         else:
         ...             node.send(TwoIntsC(x=a, y=a))
     """
-    def decorator(a_func: InteractiveFuncType):
+    if len(in_params) == 0 and out_type == Void:
+        raise DeltaIOError('Interactive node must have either an input '
+                           'or an output. Otherwise it may freeze '
+                           'the runtime simulator.')
+
+    def decorator(a_func: Callable[[PythonNode], None]):
         proc = InteractiveProcess(a_func,
                                   in_params,
                                   out_type,
